@@ -4,25 +4,28 @@ import os
 import shutil
 import filecmp
 import shlex
-from xdg.DesktopEntry import DesktopEntry
+from xdg import DesktopEntry
+from desktop_entry_lib import DesktopEntry as JdDesktopEntry
+from desktop_entry_lib import DesktopAction as JdDesktopAction
 
 import dataclasses
-from ..lib.constants import APP_ID, TMP_DIR
-from ..lib import terminal
+from ..models.Settings import Settings
 from ..models.AppListElement import AppListElement, InstalledStatus
-from ..lib.async_utils import _async, idle
-from ..lib.json_config import save_config_for_app, read_config_for_app
-from ..lib.utils import get_giofile_content_type, get_gsettings, gio_copy, get_file_hash, \
+from ..lib.constants import TMP_DIR
+from ..lib import terminal
+from ..lib.async_utils import idle
+from ..lib.json_config import read_config_for_app, remove_update_config
+from ..lib.utils import get_giofile_content_type, gio_copy, get_file_hash, \
     remove_special_chars, get_random_string, show_message_dialog, get_osinfo, extract_terminal_arguments
 from ..models.Models import AppUpdateElement, InternalError, DownloadInterruptedException
 from typing import Optional, List, TypedDict
-from gi.repository import GLib, Gtk, Gdk, Gio, Adw
+from gi.repository import GLib, Gtk, Gdk, Gio
 from enum import Enum
 
 
 class ExtractedAppImage():
     extraction_folder: str
-    desktop_entry: Optional[DesktopEntry]
+    desktop_entry: Optional[DesktopEntry.DesktopEntry]
     appimage_file: Gio.File
     desktop_file: Optional[Gio.File]
     icon_file: Optional[Gio.File]
@@ -48,7 +51,7 @@ class AppImageListElement():
     is_updatable_from_url = False
     env_variables: List[str] = dataclasses.field(default_factory=lambda: [])
     exec_arguments: str = ''
-    desktop_entry: Optional[DesktopEntry] = None
+    desktop_entry: Optional[DesktopEntry.DesktopEntry] = None
     update_logic: Optional[AppImageUpdateLogic] = None
     architecture: Optional[AppImageArchitecture] = None
     updating_from: Optional[any] = None # AppImageListElement
@@ -77,17 +80,17 @@ class AppImageProvider():
         self.desktop_exec_codes = ["%f", "%F",  "%u",  "%U",  "%i",  "%c", "%k"]
         logging.info(f'Activating {self.name} provider')
 
-
         self.general_messages = []
         self.update_messages = []
 
         self.extraction_folder = os.path.join(TMP_DIR, 'appimages')
         self.user_desktop_files_path = os.path.join(GLib.get_home_dir(), '.local', 'share', 'applications')
         self.user_local_share_path = os.path.join(GLib.get_home_dir(), '.local', 'share')
+    desk_entry_section_regex = re.compile(r'\[Desktop Entry\][\s\S]*?(?=\n\[)', flags=re.MULTILINE)
 
     def list_installed(self) -> list[AppImageListElement]:
         default_folder_path = self._get_appimages_default_destination_path()
-        manage_from_outside = get_gsettings().get_boolean('manage-files-outside-default-folder')
+        manage_from_outside = Settings.settings.get_boolean('manage-files-outside-default-folder')
         output = []
 
         if not os.path.exists(self.user_desktop_files_path):
@@ -99,9 +102,10 @@ class AppImageProvider():
 
             try:
                 if os.path.isfile(gfile.get_path()) and get_giofile_content_type(gfile) == 'application/x-desktop':
-                    entry = DesktopEntry(filename=gfile.get_path())
+                    entry = DesktopEntry.DesktopEntry(filename=gfile.get_path())
                     exec_location = entry.getTryExec()
                     exec_command_data = extract_terminal_arguments(entry.getExec())
+                    exec_arguments = ' '.join(exec_command_data['arguments'])
 
                     if os.path.isfile(exec_location):
                         exec_gfile = Gio.File.new_for_path(exec_location)
@@ -114,18 +118,18 @@ class AppImageProvider():
                                 name=entry.getName(),
                                 desktop_file_path=gfile.get_path(),
                                 description=entry.getComment(),
-                                version=entry.get('X-AppImage-Version'),
+                                version=self._get_app_version(None, desktop_entry=entry),
                                 installed_status=InstalledStatus.INSTALLED,
                                 file_path=exec_location,
                                 provider=self.name,
                                 desktop_entry=entry,
                                 trusted=True,
                                 external_folder=(not exec_in_defalut_folder),
-                                exec_arguments=shlex.join(exec_command_data['arguments']),
+                                exec_arguments=exec_arguments,
                                 env_variables=exec_command_data['env_vars'],
                             )
 
-                            list_element.architecture = self.get_elf_arch(list_element)
+                            list_element.architecture = None
 
                             output.append(list_element)
                         else:
@@ -178,7 +182,7 @@ class AppImageProvider():
         
         return ''
 
-    def refresh_title(self, el: AppImageListElement):
+    def refresh_data(self, el: AppImageListElement):
         if el.desktop_entry:
             el.name = el.desktop_entry.getName()
         
@@ -187,7 +191,10 @@ class AppImageProvider():
             el.name = extracted.desktop_entry.getName()
             el.version = el.desktop_entry.get('X-AppImage-Version')
 
-    def uninstall(self, el: AppImageListElement, force_delete=False):
+    def refresh_arch(self, el: AppImageListElement):
+        el.architecture = self.get_elf_arch(el)
+
+    def uninstall(self, el: AppImageListElement, force_delete=False, remove_configuration=True):
         logging.info(f'Removing {el.file_path}')
 
         gf = Gio.File.new_for_path(el.file_path)
@@ -211,6 +218,9 @@ class AppImageProvider():
         if '/' in icon and os.path.isfile(icon):
             os.remove(icon)
 
+        if remove_configuration:
+            remove_update_config(el)
+
         el.set_installed_status(InstalledStatus.NOT_INSTALLED)
 
     def search(self, query: str) -> list[AppListElement]:
@@ -279,7 +289,7 @@ class AppImageProvider():
                     appimage_filename = appimage_filename.lower().replace(' ', '_')
                 
                 append_file_ext = True
-                gsettings = get_gsettings()
+                gsettings = Settings.settings
 
                 if extracted_appimage.desktop_entry and \
                     gsettings.get_boolean('exec-as-name-for-terminal-apps') and \
@@ -346,72 +356,54 @@ class AppImageProvider():
             if not os.path.exists(self.user_desktop_files_path):
                 os.makedirs(self.user_desktop_files_path)
 
-            dest_desktop_file_path = os.path.join(self.user_desktop_files_path, prefixed_filename) + '.desktop'
+            dest_desktop_file_path = f'{os.path.join(self.user_desktop_files_path, prefixed_filename)}.desktop'
             dest_desktop_file_path = dest_desktop_file_path.replace(' ', '_')
 
             # Get default exec arguments
-            exec_arguments = shlex.split(extracted_appimage.desktop_entry.getExec())[1:]
-            el.exec_arguments = shlex.join(exec_arguments)
+            term_arguments = extract_terminal_arguments(extracted_appimage.desktop_entry.getExec())
+            exec_arguments = term_arguments['arguments']
+            el.exec_arguments = ' '.join(exec_arguments)
 
-            desktop_file_content = ''
-            with open(extracted_appimage.desktop_file.get_path(), 'r') as dskt_file:
-                desktop_file_content = dskt_file.read()
+            jd_desktop_entry = JdDesktopEntry.from_file(
+                extracted_appimage.desktop_file.get_path()
+            )
 
-            escaped_exec_filepath = self._escape_exec_argument(dest_appimage_file.get_path())
-            icon_key = 'Icon=applications-other'
+            jd_desktop_entry.TryExec = dest_appimage_file.get_path()
+            jd_desktop_entry.Exec = shlex.join([dest_appimage_file.get_path(), *exec_arguments])
+
+            if el.update_logic is AppImageUpdateLogic.KEEP:
+                final_app_name = extracted_appimage.desktop_entry.getName() + f' ({version})'
+                jd_desktop_entry.Name.default_text = final_app_name
+
+            desktop_icon = f'applications-other'
             if dest_appimage_icon_file:
-                icon_key = dest_appimage_icon_file.get_path()
+                desktop_icon = f"{dest_appimage_icon_file.get_path()}"
             
-            for g in extracted_appimage.desktop_entry.groups():
-                if g == DesktopEntry.defaultGroup:
-                    exec_key = extracted_appimage.desktop_entry.get('Exec', group=g)
-                    exec_kg_arguments = shlex.split(exec_key)[1:]
+            jd_desktop_entry.Icon = desktop_icon
 
-                    exec_line = ' '.join([
-                        escaped_exec_filepath, 
-                        shlex.join(exec_kg_arguments)
-                    ])
+            for a, action in jd_desktop_entry.Actions.items():
+                a_exec_args = extract_terminal_arguments(action.Exec)
+                action.Icon = desktop_icon
+                action.Exec = shlex.join([dest_appimage_file.get_path(), *a_exec_args['arguments']])
 
-                    desktop_file_content = self._desktop_file_replace_key(desktop_file_content, g, 'Exec', exec_line)
-                    desktop_file_content = self._desktop_file_replace_key(desktop_file_content, g, 'TryExec', dest_appimage_file.get_path())
-                    desktop_file_content = self._desktop_file_replace_key(desktop_file_content, g, 'Icon', icon_key)
-                else:
-                    if extracted_appimage.desktop_entry.hasKey('Icon', group=g):
-                        desktop_file_content = self._desktop_file_replace_key(desktop_file_content, g, 'Icon', icon_key)
+            desktop_file_content = jd_desktop_entry.get_text()
 
-                    if extracted_appimage.desktop_entry.hasKey('Exec', group=g):
-                        exec_key = extracted_appimage.desktop_entry.get('Exec', group=g)
-                        exec_kg_arguments = shlex.split(exec_key)[1:]
-                        exec_line = ' '.join([
-                            escaped_exec_filepath, 
-                            shlex.join(exec_kg_arguments)
-                        ])
+            desktop_file_entry_section_match = self.desk_entry_section_regex.search(desktop_file_content)
+            original_desktop_entry_section = desktop_file_content
 
-                        desktop_file_content = self._desktop_file_replace_key(
-                            desktop_file_content, g, 'Exec', exec_line)
+            if desktop_file_entry_section_match:
+                original_desktop_entry_section = desktop_file_entry_section_match.group(0)
 
-            # generate a new app name
-            final_app_name = extracted_appimage.appimage_file.get_basename()
-            if extracted_appimage.desktop_entry:
-                final_app_name = extracted_appimage.desktop_entry.get('Name', group=DesktopEntry.defaultGroup)
+            desktop_file_entry_section = original_desktop_entry_section
+            desktop_file_entry_section = re.sub(r'^X-AppImage-Version=.*$', "", desktop_file_entry_section, flags=re.MULTILINE)
+            desktop_file_entry_section += f'\nX-AppImage-Version={version}\n'
 
-                if el.update_logic is AppImageUpdateLogic.KEEP:
-                    final_app_name += f' ({version})'
-
-            desktop_file_content = self._desktop_file_replace_key(
-                desktop_file_content,
-                group_name=DesktopEntry.defaultGroup,
-                key='X-AppImage-Version',
-                replacement=version
+            desktop_file_content = desktop_file_content.replace(
+                original_desktop_entry_section,
+                desktop_file_entry_section,
             )
 
-            final_app_name = final_app_name.strip()
-            desktop_file_content = self._desktop_file_replace_key(
-                desktop_file_content,
-                group_name=DesktopEntry.defaultGroup,
-                key='Name',
-                replacement=final_app_name
-            )
+            desktop_file_content = re.sub(r'\n\n(?!\[)', '\n', desktop_file_content)
 
             # finally, write the new .desktop file
             if (not os.path.exists(self.user_desktop_files_path)) and os.path.exists(self.user_local_share_path):
@@ -421,7 +413,7 @@ class AppImageProvider():
                 desktop_file_python_dest.write(desktop_file_content)
 
             if os.path.exists(dest_desktop_file_path):
-                el.desktop_entry = DesktopEntry(filename=dest_desktop_file_path)
+                el.desktop_entry = DesktopEntry.DesktopEntry(filename=dest_desktop_file_path)
                 el.desktop_file_path = dest_desktop_file_path
                 el.installed_status = InstalledStatus.INSTALLED
 
@@ -443,7 +435,7 @@ class AppImageProvider():
             logging.error('Appimage installation error: ' + str(e))
             raise e
 
-        if get_gsettings().get_boolean('move-appimage-on-integration'):
+        if Settings.settings.get_boolean('move-appimage-on-integration'):
             if os.path.dirname(extracted_appimage.appimage_file.get_path()) != appimages_destination_path:
                 logging.info('Deleting original appimage file from: '  + extracted_appimage.appimage_file.get_path())
                 if not extracted_appimage.appimage_file.delete(None):
@@ -470,7 +462,7 @@ class AppImageProvider():
 
         gio_copy(outdated_file, new_file)
 
-        self.uninstall(el)
+        self.uninstall(el, remove_configuration=False)
 
         el.file_path = f'{dest_path}/tmp.appimage'
         el.extracted = None
@@ -496,6 +488,7 @@ class AppImageProvider():
             raise InternalError(message='This file type is not supported')
         
         app_name: str = os.path.basename(file.get_parse_name())
+        preview_enabled = Settings.settings.get_boolean('preview-before-opening-app')
 
         el = AppImageListElement(
             name=re.sub(r'\.appimage$', '', app_name, 1, re.IGNORECASE),
@@ -506,6 +499,7 @@ class AppImageProvider():
             file_path=file.get_path(),
             desktop_entry=None,
             local_file=True,
+            trusted=(not preview_enabled)
         )
 
         el.architecture = self.get_elf_arch(el)
@@ -526,12 +520,37 @@ class AppImageProvider():
             shutil.rmtree(self.extraction_folder)
             os.makedirs(self.extraction_folder)
 
+    def update_exec_arguments(self, el:AppImageListElement, arg_string: str):
+        arg_string = arg_string.replace("\n", "")
+
+        if not el.desktop_file_path:
+            raise Exception('desktop_file_path not specified')
+    
+        desktop_file_content = ''
+        entry = DesktopEntry.DesktopEntry(filename=el.desktop_file_path)
+        with open(el.desktop_file_path, 'r') as desktop_file:
+            desktop_file_content = desktop_file.read()
+            exec_command = shlex.split(entry.getExec())[0]
+            exec_command += f' {arg_string}'
+
+            # replace executable path
+            desktop_file_content = re.sub(
+                r'^Exec=.*$',
+                f"Exec={exec_command}",
+                desktop_file_content,
+                flags=re.MULTILINE
+            )
+
+        with open(el.desktop_file_path, 'w') as desktop_file:
+            desktop_file.write(desktop_file_content)         
+
     def update_desktop_file(self, el: AppImageListElement):
         if not el.desktop_file_path:
             raise Exception('desktop_file_path not specified')
+        
+        jd_desktop_file = JdDesktopEntry.from_file(el.desktop_file_path)
 
-        entry = DesktopEntry(filename=el.desktop_file_path)
-        tryexec_command = entry.getTryExec()
+        tryexec_command = jd_desktop_file.TryExec
         exec_arguments = el.exec_arguments
         env_vars = ' '.join(el.env_variables)
 
@@ -543,36 +562,20 @@ class AppImageProvider():
 
         exec_command = ''.join([
             env_vars,
-            self._escape_exec_argument(tryexec_command),
+            shlex.quote(tryexec_command),
             exec_arguments
         ])
 
-        desktop_file_content = ''
-        with open(el.desktop_file_path, 'r') as desktop_file:
-            desktop_file_content = desktop_file.read()
+        jd_desktop_file.Exec = exec_command
+        jd_desktop_file.write_file(el.desktop_file_path)
 
-        # replace executable path
-        desktop_file_content = self._desktop_file_replace_key(
-            desktop_file_content,
-            group_name=DesktopEntry.defaultGroup,
-            key='Exec',
-            replacement=exec_command
-        )
+        el.desktop_entry = DesktopEntry.DesktopEntry(filename=el.desktop_file_path)
 
-        tmp_desktop_file = os.path.join(TMP_DIR, get_random_string() + '.desktop')
-        with open(tmp_desktop_file, 'w+') as desktop_file:
-            desktop_file.write(desktop_file_content)
-
-        terminal.host_sh(['desktop-file-validate', '--no-hints', '--no-warn-deprecated', tmp_desktop_file])
-
-        shutil.move(tmp_desktop_file, el.desktop_file_path)
-        el.desktop_entry = DesktopEntry(filename=el.desktop_file_path)
-
-    def update_from_url(self, manager, el: AppImageListElement, status_cb: callable) -> AppImageListElement:
+    def update_from_url(self, manager, el: AppImageListElement, status_cb: callable) -> AppImageListElement | None:
         try:
             update_file_path, f_hash = manager.download(status_cb)
         except DownloadInterruptedException as de:
-            return el
+            return None
         except Exception as e:
             raise e
 
@@ -593,45 +596,6 @@ class AppImageProvider():
         return list_element
 
     # Private methods
-    def _escape_exec_argument(self, arg: str) -> str:
-        """
-        Escape an input string according to the Exec key quoting rules
-        from the freedesktop.org Desktop Entry Specification.
-
-        Escapes:
-        - Double quote (")
-        - Backtick (`)
-        - Dollar sign ($)
-        - Backslash (\\)
-
-        Returns the escaped string, enclosed in double quotes if needed.
-        """
-        reserved_chars = set(' \t\n"\'\\><~|&;$*?#()`')
-
-        def escape_inner(s: str) -> str:
-            s = s.replace("\\", "\\\\")  # Escape backslash first
-            s = s.replace('"', r'\"')
-            s = s.replace('`', r'\`')
-            s = s.replace('$', r'\$')
-            return s
-
-        if any(ch in reserved_chars for ch in arg):
-            return f'"{escape_inner(arg)}"'
-        else:
-            return arg
-
-    def _desktop_file_replace_key(self, content: str, group_name: str, key: str, replacement: str):
-        pattern = rf'(\[{group_name}\][\s\S]*?)^{key}=.*$'
-
-        if re.search(pattern, content, flags=re.MULTILINE):
-            replacement = rf'\1{key}={replacement}'
-        else:
-            pattern = rf'\[{group_name}\].*$'
-            replacement = f'[{group_name}]\n{key}={replacement}'
-
-
-        return re.sub(pattern, replacement, content, flags=re.MULTILINE, count=1)
-
     def _run_filepath(self, el: AppImageListElement):
         is_nixos = re.search(r"^NAME=NixOS$", get_osinfo(), re.MULTILINE) != None
 
@@ -740,6 +704,7 @@ class AppImageProvider():
                 appimage_offset = terminal.sandbox_sh(['/usr/share/gearlever/get_appimage_offset', file.get_path()])
 
                 try:
+                    terminal.sandbox_sh(['unsquashfs', '-o', appimage_offset, '-l', file.get_path()])
                     terminal.sandbox_sh(['unsquashfs', '-o', appimage_offset, '-d', squashfs_root_folder, file.get_path()])
                 except Exception as e:
                     logging.error('Extraction with unsquashfs failed')
@@ -763,7 +728,7 @@ class AppImageProvider():
 
         icon_file: Optional[Gio.File] = None
         desktop_file: Optional[Gio.File] = None
-        desktop_entry: Optional[DesktopEntry] = None
+        desktop_entry: Optional[DesktopEntry.DesktopEntry] = None
 
         # hash file
         md5_hash = get_file_hash(file)
@@ -794,7 +759,7 @@ class AppImageProvider():
 
                 desktop_entry_icon = None
                 if desktop_file:
-                    desktop_entry = DesktopEntry(desktop_file.get_path())
+                    desktop_entry = DesktopEntry.DesktopEntry(desktop_file.get_path())
                     desktop_entry_icon = desktop_entry.getIcon()
                     desktop_entry_icon = re.sub(r"\.(png|svg)$", '', desktop_entry_icon)
 
@@ -873,16 +838,18 @@ class AppImageProvider():
         return result
 
     def _get_appimages_default_destination_path(self) -> str:
-        folder = get_gsettings().get_string('appimages-default-folder')
+        folder = Settings.settings.get_string('appimages-default-folder')
         return re.sub(r'^~', GLib.get_home_dir(), folder)
 
-    def _get_app_version(self, extracted_appimage: ExtractedAppImage):
-        version = None
+    def _get_app_version(self, extracted_appimage: Optional[ExtractedAppImage], desktop_entry: Optional[DesktopEntry] = None):
+        if not desktop_entry:
+            desktop_entry = extracted_appimage.desktop_entry
 
-        if extracted_appimage.desktop_entry:
-            version = extracted_appimage.desktop_entry.get('X-AppImage-Version')
+        version = desktop_entry.get(
+            'X-AppImage-Version'
+        )
 
-        if not version:
+        if (not version) and extracted_appimage:
             version = extracted_appimage.md5[0:6]
 
         return version
